#!/bin/bash
#
# uninstall.sh
#
# Copyright (C) 2011-2014 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.


VERSION="2.2"

#-------------------------------------------------------------------------------
#				Script functions
#-------------------------------------------------------------------------------
usage()
{
	echo "Usage: ${0##*/} -d <directory> [-s <directory>] [-hV]"
	echo "-d <directory>            uninstallation directory of Warden server"
	echo "-s <directory>            directory for symlinks to Warden server control scripts (optional)"
	echo "-h                        print this help"
	echo "-V                        print script version number and exit"
	echo
	echo "Example: $ ${0##*/} -d /opt/warden-server -s /usr/local/bin"
	echo
	echo "For more information about uninstallation process, see README file (section Uninstallation)."
	echo
	exit 0
}


version()
{
	echo "${0##*/} - current version is $VERSION"
	exit 0
}


err()
{
	echo "FAILED!"
	echo -n "Error: " && cat "$err"
	rm -f "$err"
	rm -rf "$backup"
	echo
	echo "Uninstallation of $package_version package FAILED!!!"
	exit 1
}


errClean()
{
	echo "FAILED!"
	echo -n "Error: " && cat "$err"
	echo " -> Reverting changes of warden server package ... "
	rm -rf "${basedir}/"* > /dev/null 2>&1
	cp -R "${backup}/"* "$basedir"
	if [[ ! -z "$symbin" ]] && [[ -d "$symbin" ]] && [[ -w "$symbin" ]]; then
		for file in "${bin}/"*
		do
			ln -s "${bin}/${file##*/}" "${symbin}/${file##*/}"
		done
	fi
	rm -f "$err"
	rm -rf "$backup"
	echo
	echo "Uninstallation of $package_version package FAILED!!!"
	exit 1
}


paramsChck()
{
	if [ -z "$basedir" ]; then
		echo "Parameter -d <directory> is not set!"
		exit 1
	fi
}


#-------------------------------------------------------------------------------
#				Uninstallation functions
#-------------------------------------------------------------------------------

wardenDirChck()
{
	echo -n "Checking Warden server directory ... "
	if [[ ! -d "$basedir" ]] && ; then
		echo "FAILED!"
		echo "Warden server package is not installed!"
		exit 1
	elif [[ ! -w "$basedir" ]]; then
		echo "FAILED!"
		echo "You don't have permission to Warden server directory!"
		exit 1
	else
		echo "OK"
	fi
}


getPackageVersion()
{
	echo -n "Checking Warden server version ... "
	old_package_version_file=$(find "$basedir" -name package_version)
	if [ -f "$old_package_version_file" ]; then
		package_version=$(<"$old_package_version_file")
	else
		package_version="unknown"
	fi
	echo "OK"
}


createBackup()
{
	echo -n "Creating Warden server backup ... "
	[[ -d "$backup" ]] && rm -rf "$backup" || mkdir "$backup"
	if cp -R "${basedir}/"* "$backup" 2> "$err"; then
		echo "OK"
	else
		err
	fi
}


uninstallWardenServer()
{
	echo -n "Uninstalling $package_version package ... "
	cp "${doc}/UNINSTALL" "$uninstall"
	manifest=$(find "$basedir" -name MANIFEST) || err
	for file in $(<"$manifest")
	do
		rm -f "${basedir}/$file" 2> "$err" || errClean
		directory="${file%/*}"
		rmdir --ignore-fail-on-non-empty "${basedir}/$directory" 2> "$err" || errClean
	done
	rm -f "${basedir}/uninstall.sh"
	rmdir --ignore-fail-on-non-empty "$basedir"
	echo "OK"
}


deleteSymlinks()
{
	echo -n "Deleting symlinks from $symbin ..."
	for file in "$symbin/"*
	do
		rm -f "${symbin}/${file##*/}" 2> /dev/null
	done
	echo "OK"
}


#-------------------------------------------------------------------------------
#				MAIN
#-------------------------------------------------------------------------------
# read input
while getopts "d:s:Vh" options; do
	case "$options" in
		d ) basedir="$OPTARG";;
		s ) symbin="$OPTARG";;
		h ) usage;;
		V ) version;;
		* ) usage;;
	esac
done

# remove last char (slash) from name of directories
[[ "$basedir" == */ ]] && basedir="${basedir%?}"

# params test
paramsChck

# set variables
bin="${basedir}/bin"
etc="${basedir}/etc"
doc="${basedir}/doc"
err="/tmp/warden-err"
uninstall="/tmp/UNINSTALL.warden"
backup="/tmp/warden-backup"

echo "Uninstallation of Warden server package ..."
echo
echo "------------------------- Uninstallation process --------------------------------"

# check Warden server directory if exists
wardenDirChck

# obtain version of existing Warden server (if exists)
getPackageVersion

# make backup of currently installed warden-server package
createBackup

# uninstall Warden server
uninstallWardenServer || exit 1

# delete symlinks from $symbin (if -s option is set)
[[ ! -z "$symbin" ]] && [[ -d "$symbin" ]] && [[ -w "$symbin" ]] && deleteSymlinks

echo
echo "Please follow post-uninstallation steps in $uninstall!"
echo
echo "Uninstallation of $package_version package was SUCCESSFUL!!!"
echo

# cleanup section
rm -r "$err"
rm -rf "$backup"

exit 0
